package BIDM::Attribute;

=head1 NAME

BIDM::Attribute

=head1 DESCRIPTION

Representation of a single attribute in the BIDM data model.

=cut

use strict;


use BIDM::MemberMixin;
use vars qw(@ISA);

@ISA = qw(BIDM::MemberMixin);


sub new
{
    my($class, $name) = @_;
    my($self);

    $self = {
	name => $name,
	description => '',
	is_optional => 0,
	is_single => 0,
	constraints => '',
	data_type => '',
    };

    bless($self, $class);

    return $self;
}

sub get_object_attribute
{
    my($self, $obj) = @_;
    my($name, $value);

    $name = $self->name();

    if (defined($value = $obj->attributes()->{$name}))
    {
	if ($self->is_single())
	{
	    return wantarray ? ($value) : $value;
	}
	else
	{
	    if (wantarray)
	    {
		return @$value;
	    }
	    else
	    {
		return $value->[0];
	    }
	}
    }
    else
    {
	return wantarray ? () : undef;
    }
}

#
# Set the value of this attribute in the given object. Check for
# redefinition of single-valued fields, and handle the listwise
# defintion of multi-valued fields.
#
sub set_object_attribute
{
    my($self, $obj, $value) = @_;

    if ($self->is_single)
    {
	$self->set_single($obj, $value);
    }
    else
    {
	$self->set_multiple($obj, $value);
    }
}

sub set_single
{
    my($self, $obj, $value) = @_;
    my($name);

    $name = $self->name();
    if (defined($obj->attributes()->{$name}))
    {
	die "Single-valued attribute $name already set in object ", $obj->class_name(), "\n";
    }

    $obj->attributes()->{$name} = $value;
}

sub set_multiple
{
    my($self, $obj, $value) = @_;
    my($name, $list);

    $name = $self->name();
    $list = $obj->attributes()->{$name};
    if (!defined($list))
    {
	$list = [];
	$obj->attributes()->{$name} = $list;
    }

    push(@$list, $value);
}

sub load_from_fh
{
    my($self, $fh) = @_;
    my($key, $val);
    
    while (<$fh>)
    {
	chop;
	s/^\s+//;
	s/\s+$//;
	next if /^#/;
	last if /^%end/;

	($key, $val) = split(/\s+/, $_, 2);

	if (!defined($self->{$key}))
	{
	    die "Invalid attribute $key\n";
	}
#	print "   $key = $val\n";
	$self->$key($val);
    }
}

sub name { shift->_elem('name', @_); }
sub type { shift->_elem('type', @_); }
sub description { shift->_elem('description', @_); }
sub is_optional { shift->_elem('is_optional', @_); }
sub is_single { shift->_elem('is_single', @_); }
sub constraints { shift->_elem('constraints', @_); }
sub data_type { shift->_elem('data_type', @_); }


1;
