package BIDM::Tag;

=head1 NAME

BIDM::Tag
    
=head1 DESCRIPTION

Representation of a single BIDM tag.
   
=cut

use strict;

use BIDM::Tag::Link;
use BIDM::Tag::Meta;
use BIDM::MemberMixin;

use vars qw(@ISA);

@ISA = qw(BIDM::MemberMixin);

sub new
{
    my($class, $type, $attrs) = @_;
    my($self);

    $self = {
	attributes => $attrs,
	type => $type,
    };

    bless($self, $class);

    return $self;
}

sub create_from_tag
{
    my($tag, $attrs) = @_;
    my($bidm_tag);

    if ($tag eq 'meta')
    {
	$bidm_tag = new BIDM::Tag::Meta($attrs);
	return $bidm_tag;
    }
    elsif ($tag eq 'link')
    {
#	print "Create link\n";
	$bidm_tag = new BIDM::Tag::Link($attrs);
#	print "Created tag is $bidm_tag\n";
#	$bidm_tag->print();
	return $bidm_tag;
    }
    else
    {
	die "Unknown tag `$tag' in BIDM::Tag::create_from_tag\n";
    }
}

sub store_in_db
{
    my($self, $db, $id, $tagnum) = @_;
    my($k, $v, $t);

    $t = $self->{type};

    while (($k, $v) = each(%{$self->{attributes}}))
    {
	$k =~ s/'/\\'/g;
	$v =~ s/'/\\'/g;
	$db->query("insert into Tag values ('$id', '$t', $tagnum, '$k', '$v')");
    }
}

sub print
{
    my($self, $fh) = @_;
    my($k, $v);
    
    $fh = \*STDOUT unless defined($fh);

    print $fh "  $self:\n";

    while (($k, $v) = each(%{$self->{attributes}}))
    {
	print $fh "    $k = $v\n";
    }
}

1;
